package com.huigou.topsun.sap.dull.application.impl;

import com.huigou.cache.DictUtil;
import com.huigou.data.query.model.QueryDescriptor;
import com.huigou.data.query.model.QueryModel;
import com.huigou.topsun.base.coderule.application.CodeRuleApplication;

import com.huigou.topsun.sap.dull.application.SapDullApplication;
import com.huigou.topsun.sap.dull.domain.SapDull;
import com.huigou.topsun.sap.dull.domain.query.SapDullQueryRequest;
import com.huigou.topsun.sap.dull.repository.SapDullRepository;
import com.huigou.uasp.bmp.common.BizBillStatus;
import com.huigou.uasp.bmp.doc.attachment.application.AttachmentApplication;
import com.huigou.uasp.bpm.FlowBroker;
import com.huigou.util.ClassHelper;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.DelegateTask;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @Auther: Lxh
 * @Date: 2024/03/28/19:55
 * @Description:
 */
@Service("sapDullApplication")
public class SapDullApplicationImpl extends FlowBroker implements SapDullApplication {
    @Autowired
    private SapDullRepository sapDullRepository;

   // @Autowired
   // private SapDullItemRepository sapDullItemRepository;
    @Autowired
    private AttachmentApplication attachmentApplication;
    @Autowired
    private CodeRuleApplication codeRuleApplication;
    @Override
    public Map<String, Object> getDullList(SapDullQueryRequest queryRequest) {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "sapDull");
        QueryModel queryModel = this.sqlExecutorDao.getQueryModel(queryDescriptor, queryRequest);
        if (queryRequest.getStatus() != null) {
            queryModel.addCriteria(" and t.status =:status");
            queryModel.putParam("status", queryRequest.getStatus());
        }
        if (queryRequest.getBillCode() != null) {
            queryModel.addCriteria(" and t.bill_code =:billCode");
            queryModel.putParam("billCode", queryRequest.getBillCode());
        }
        if (queryRequest.getMatnr() != null) {
            queryModel.addCriteria(" and t.matnr like:matnr");
            queryModel.putLikeParam("matnr", queryRequest.getMatnr());
        }
        if (queryRequest.getMatnrName() != null) {
            queryModel.addCriteria(" and t.matnr_name like:matnr");
            queryModel.putLikeParam("matnr", queryRequest.getMatnrName());
        }
        queryModel.putDictionary("status", DictUtil.getDictionary("bizBillStatus"));
        queryModel.addCriteria(" order by bill_code desc ");
        Map<String, Object> map = this.sqlExecutorDao.executeSlicedQuery(queryModel);
        return map;
    }

    @Override
    public SapDull getDullById(String id) {
        return sapDullRepository.findOne(id);
    }

    @Override
    public void deleteByIds(List<String> ids) {
        ids.forEach(id->{
             sapDullRepository.delete(id);

        });
    }

    @Override
    protected void onBeforeComplete(DelegateTask delegateTask) {
        super.onBeforeComplete(delegateTask);
        String bizId = delegateTask.getExecution().getProcessBusinessKey();
        if (this.isApplyProcUnit(delegateTask) && this.getApprovalParameter().isAdvanceProcessAction()) {
            SapDull sapDull = sapDullRepository.findOne(bizId);
            sapDull.setStatusId(BizBillStatus.APPROVING.getId());
            sapDullRepository.save(sapDull);
        }
    }

    @Override
    protected void onEnd(DelegateExecution delegateExecution) {
        super.onEnd(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        BizBillStatus status = approvePassed() ? BizBillStatus.COMPLETED : BizBillStatus.ABORTED;
        SapDull Dull = sapDullRepository.findOne(bizId);
        Dull.setStatusId(status.getId());
        sapDullRepository.save(Dull);
    }

    @Override
    protected void onAbortProcessInstance(DelegateExecution delegateExecution) {
        super.onAbortProcessInstance(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        SapDull Dull = sapDullRepository.findOne(bizId);
        Dull.setStatusId(BizBillStatus.ABORTED.getId());
        sapDullRepository.save(Dull);
    }

    @Override
    protected void onBack(DelegateTask delegateTask, String destActivityId) {
        super.onBack(delegateTask, destActivityId);
        if ("Apply".equalsIgnoreCase(destActivityId)) {
            String bizId = delegateTask.getExecution().getProcessBusinessKey();
            SapDull Dull = sapDullRepository.findOne(bizId);
            Dull.setStatusId(BizBillStatus.APPLYING.getId());
            sapDullRepository.save(Dull);
        }
    }

    @Override
    protected Map<String, Object> getProcessBizParams(String bizId) {
        return ClassHelper.toMap(sapDullRepository.getOne(bizId));
    }

    @Override
    protected String saveBizAndApprovalData() {
        super.saveBizAndApprovalData();
        SapDull Dull = getBizEntity(SapDull.class);
        if (Dull.isNew()) {
            Dull.setStatusId(BizBillStatus.APPLYING.getId());
        } else {
            Dull = (SapDull) commonDomainService.loadAndFillinProperties(Dull);
        }
        Dull = sapDullRepository.save(Dull);


        return Dull.getId();
    }



}
