package com.huigou.topsun.sap.exceptionReport.application.impl;

import com.huigou.cache.DictUtil;
import com.huigou.data.domain.model.CommonDomainConstants;
import com.huigou.data.query.model.QueryDescriptor;
import com.huigou.data.query.model.QueryModel;
import com.huigou.topsun.common.CommonUtils;
import com.huigou.topsun.sap.exceptionReport.application.ExceptionReportApplication;
import com.huigou.topsun.sap.exceptionReport.application.ExceptionReportDutyPersonApplication;
import com.huigou.topsun.sap.exceptionReport.application.ExceptionReportItemApplication;
import com.huigou.topsun.sap.exceptionReport.domain.ExceptionReport;
import com.huigou.topsun.sap.exceptionReport.domain.ExceptionReportDutyPerson;
import com.huigou.topsun.sap.exceptionReport.domain.ExceptionReportItem;
import com.huigou.topsun.sap.exceptionReport.domain.query.ExceptionReportQueryRequest;
import com.huigou.topsun.sap.exceptionReport.repository.ExceptionReportRepository;
import com.huigou.topsun.sap.permit.domain.SapPermit;
import com.huigou.topsun.sap.permit.domain.SapPermitItem;
import com.huigou.uasp.bmp.common.BizBillStatus;
import com.huigou.uasp.bpm.FlowBroker;
import com.huigou.util.*;
import freemarker.template.Template;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.DelegateTask;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.web.servlet.view.freemarker.FreeMarkerConfigurer;

import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Auther: xin.lu
 * @Date: 2024/03/20/19:36
 * @Description:
 */
@Service("exceptionReportApplication")
public class ExceptionReportApplicationImpl extends FlowBroker implements ExceptionReportApplication {
    @Autowired
    private ExceptionReportRepository exceptionReportRepository;

    @Autowired
    private ExceptionReportItemApplication exceptionReportItemApplication;

    @Autowired
    private ExceptionReportDutyPersonApplication exceptionReportDutyPersonApplication;
    @Autowired
    private FreeMarkerConfigurer freemarkerConfigurer;
    @Override
    public Map<String, Object> slicedExceptionReportList(ExceptionReportQueryRequest queryRequest) {
        QueryDescriptor queryDescriptor = this.sqlExecutorDao.getQuery(QUERY_XML_FILE_PATH, "exceptionReport");
        QueryModel queryModel = this.sqlExecutorDao.getQueryModel(queryDescriptor, queryRequest);
        queryModel.putDictionary("status", BizBillStatus.getMap());
        return this.sqlExecutorDao.executeSlicedQuery(queryModel);
    }

    @Override
    public ExceptionReport findById(String id) {
        return exceptionReportRepository.findOne(id);
    }

    @Override
    public void deleteById(String id) {
        exceptionReportRepository.delete(id);
    }

    @Override
    public ExceptionReport saveExceptionReport(ExceptionReport exceptionReport) {
        return exceptionReportRepository.save(exceptionReport);
    }

    @Transactional
    private void updateStatus(String bizId, BizBillStatus status) {
        Assert.hasText(bizId, CommonDomainConstants.ID_NOT_BLANK);
        this.commonDomainService.updateStatus(ExceptionReport.class, bizId, status.getId());
    }

    /**
     * 保存业务数据
     */
    @Override
    protected String saveBizAndApprovalData() {
        super.saveBizAndApprovalData();
        ExceptionReport exceptionReport = this.getBizEntity(ExceptionReport.class);
        List<ExceptionReportItem> exceptionReportItems = this.getBizEntities(ExceptionReportItem.class, "exceptionReportItems");
        List<ExceptionReportDutyPerson> exceptionReportDutyPersonList = this.getBizEntities(ExceptionReportDutyPerson.class, "personList");
        exceptionReport = this.saveExceptionReport(exceptionReport);
        this.exceptionReportItemApplication.saveExceptionReportItemList(exceptionReport.getId(),exceptionReportItems);
        this.exceptionReportDutyPersonApplication.saveExceptionReportDutyPersonList(exceptionReport.getId(),exceptionReportDutyPersonList);
        return exceptionReport.getId();
    }

    /**
     * 获取流程参数
     */
    @Override
    protected Map<String, Object> getProcessBizParams(String bizId) {
        return ClassHelper.toMap(exceptionReportRepository.findOne(bizId));
    }

    /**
     * 删除流程实例事件
     */
    @Override
    protected void onDeleteProcessInstance(DelegateExecution delegateExecution) {
        String bizId = delegateExecution.getProcessBusinessKey();
        deleteApplyAcceptance(bizId);
    }

    /**
     * 终止流程事件
     */
    @Override
    protected void onAbortProcessInstance(DelegateExecution delegateExecution) {
        super.onAbortProcessInstance(delegateExecution);
        String bizId = delegateExecution.getProcessBusinessKey();
        updateStatus(bizId, BizBillStatus.ABORTED);
    }


    /**
     * 流程结束事件
     */
    @Override
    protected void onEnd(DelegateExecution delegateExecution) {
        super.onEnd(delegateExecution);
        BizBillStatus status = BizBillStatus.COMPLETED;
        String bizId = delegateExecution.getProcessBusinessKey();
        ExceptionReport exceptionReport = this.findById(bizId);
        exceptionReport.setStatusId(status.getId());
        this.saveExceptionReport(exceptionReport);
    }

    /**
     * 流程撤销事件
     **/
    @Override
    protected void onRecallProcessInstance(DelegateExecution delegateExecution) {
        String bizId = delegateExecution.getProcessBusinessKey();
        // 修改单据状态为申请
        updateStatus(bizId, BizBillStatus.APPLYING);
    }

    /**
     * 任务完成前执行
     */
    @Override
    protected void onBeforeComplete(DelegateTask delegateTask) {
        super.onBeforeComplete(delegateTask);
        String bizId = delegateTask.getExecution().getProcessBusinessKey();
        if (this.isApplyProcUnit(delegateTask) && this.getApprovalParameter().isAdvanceProcessAction()) {
            updateStatus(bizId, BizBillStatus.APPROVING);
        }
    }

    /**
     * 回退
     */
    @Override
    protected void onBack(DelegateTask delegateTask, String destActivityId) {
        super.onBack(delegateTask, destActivityId);
        String bizId = delegateTask.getExecution().getProcessBusinessKey();
        if (destActivityId.equalsIgnoreCase("apply")) {
            updateStatus(bizId, BizBillStatus.APPLYING);
        }
    }

    /**
     * 回收事件
     **/
    @Override
    protected void onWithdraw(DelegateTask delegateTask, String destActivityId) {
        super.onWithdraw(delegateTask, destActivityId);
        String bizId = delegateTask.getExecution().getProcessBusinessKey();
        if (destActivityId.equalsIgnoreCase("apply")) {
            updateStatus(bizId, BizBillStatus.APPLYING);
        }
    }


    @Transactional
    public void deleteApplyAcceptance(String id) {
        Assert.hasText(id, CommonDomainConstants.ID_NOT_BLANK);
        exceptionReportRepository.delete(id);
    }


    public ByteArrayOutputStream printInfo(HttpServletResponse response, String id) {

        ExceptionReport exceptionReport=this.exceptionReportRepository.findOne(id);
        List<ExceptionReportItem> itemList=exceptionReportItemApplication.findByExceptionReportId(id);
        List<ExceptionReportDutyPerson> dutyList=exceptionReportDutyPersonApplication.findByExceptionReportId(id);
        if ("in".equals(exceptionReport.getExceptionType())){
            exceptionReport.setExceptionType("内部");
        } else{
            exceptionReport.setExceptionType("外部");
        }
        String sql="select s.code,w.handler_name,w.handler_id,w.dept_name,w.full_id,w.HANDLED_DATE  FROM WF_PROCUNITHANDLER w left join sa_oporg s on s.id=w.HANDLER_ID";
        sql+="  where  biz_id='"+id+"' order  by w.HANDLED_DATE desc ";
        List<Map<String, Object>> mapList=sqlExecutorDao.queryToListMap(sql);
        Map<String ,Object> data = new HashMap<>();
        if  (mapList.size()>0){
           String lastAuditPerson=mapList.get(0).get("handlerName").toString();
            data.put("lastAuditPerson",lastAuditPerson);
        }
        if  (mapList.size()>1) {
            String preLastAuditPerson = mapList.get(1).get("handlerName").toString();
            data.put("preAuditPerson",preLastAuditPerson);
        }

        data.put("head",exceptionReport);
        data.put("itemList",itemList);
        data.put("dutyList",dutyList);

        //Document document = null;
        ByteArrayOutputStream byteArrayOutputStream = null;
        try {
            response.reset();
            response.setHeader("Content-Type", "application/pdf-stream");
            response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode("异常单据.pdf", "UTF-8"));
           /* String randoms=CommonUtil.createGUID().substring(0,10);
            File  fileDir=new File("bar");
            if(!fileDir.exists()) {
               fileDir.mkdirs();
            }*/
           /* File file = new File("bar-"+permit.getBillCode()+".png");
            if(!file.exists()) {
                String IMG_TYPE_PNG = "image/png";
                data.put("barCode", CommonUtils.genBarCode128(permit.getBillCode(), IMG_TYPE_PNG, file));
            } else {
                data.put("barCode",file);
            }*/
            File  fileb = new File("baoshen.png");
            data.put("baoshen",fileb);
            Template template = freemarkerConfigurer.getConfiguration().getTemplate("exceptionReport.ftl");
            byteArrayOutputStream = CommonUtils.createPDF(data,template);

        }catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException("PDF生成失败！");
        } finally {

        }
        return  byteArrayOutputStream;
    }

}
